
class Cut(object):
    """
    Class used to represent a cutpoint in a range, such that any range can
    be represented by 2 Cuts
    """
    def __init__(self, theType, aboveAll=False, belowAll=False, point = None,
                 below = False):
        """ Instantiates a cut point

        Parameters
        ----------
        theType : type
            Specify the most inclusive type that can be used for comparison
        aboveAll : boolean
            Specifies that cut point is above all possible values
            within the domain
        belowAll : boolean
            Specifies that cut point is below all possible values
            within the domain
        point : instance of the domain
            A specific point in the domain where the cut should
            occur
        below : boolean
            If true, cut point is infinitesimally below specified point.
            Otherwise, is infinitesimally above specified point 
            
        Raises
        ------
        ValueError
            If input is invalid
        """
        self.theType = theType
        self.aboveAll = False
        self.belowAll = False
        self.point = None
        self.below = False
        # Validate input
        if point is None:
            if not any((aboveAll, belowAll)):
                raise ValueError("Must specify a type of cut point")
            elif all((aboveAll, belowAll)):
                raise ValueError("Cannot be both aboveAll and belowAll")
            else:
                # Correct input
                self.aboveAll = aboveAll
                self.belowAll = belowAll
        else:
            if any((aboveAll, belowAll)):
                raise ValueError("Cannot be both point and above/below all")
            elif not isinstance(point, theType):
                raise ValueError("Point must be instance of theType")
            else:
                self.point = point
                self.below = below
    def _validate_query_pt(self, pt):
        if not isinstance(pt, self.theType):
            raise ValueError("Type is not compatible with cutpoint type")
        return True
    def __hash__(self):
        if self.belowAll:
            return hash(self.theType)*31-hash(None)
        elif self.aboveAll:
            return hash(self.theType)*31+hash(None)
        elif self.below:
            return hash(self.theType)*31-hash(self.point)
        else:
            return hash(self.theType)*31+hash(self.point)
    def __repr__(self):
        if self.belowAll:
            return "Cut(Below all %s)" % str(self.theType)
        elif self.aboveAll:
            return "Cut(Above all %s)" % str(self.theType)
        elif self.below:
            return "Cut(Below %s)" % str(self.point)
        else:
            return "Cut(Above %s)" % str(self.point)
    def __cmp__(self, other):
        if self == other:
            return 0
        elif self < other:
            return -1
        elif self > other: return 1
    def __eq__(self, other):
        """ Returns whether Cuts are at EXACT same place """
        if not isinstance(other, Cut):
            return False
        elif self.aboveAll:
            return other.aboveAll
        elif self.belowAll:
            return other.belowAll
        elif (self.point is not None) and (other.point is not None):
            return ((self.point == other.point) and (self.below == other.below))
        else:
            return False
    def __ne__(self, other):
        return not self.__eq__(other)
    def __lt__(self, other):
        """ Returns whether cutpoint is less than a specified value """
        if isinstance(other, Cut):
            if self.belowAll:
                return True
            elif self.aboveAll:
                return False
            elif other.belowAll:
                return False
            elif other.aboveAll:
                return True
            else:
                if self.point < other.point:
                    return True
                elif self.point == other.point and self.below and \
                  not other.below:
                    return True
                else:
                    return False
        else:
            return self.isLessThan(other)
    def __gt__(self, other):
        """ Returns whether cutpoint is greater than a specified value """
        if isinstance(other, Cut):
            if self.belowAll:
                return False
            elif self.aboveAll:
                return True
            elif other.belowAll:
                return True
            elif other.aboveAll:
                return False
            else:
                if self.point > other.point:
                    return True
                elif self.point == other.point and not self.below and \
                  other.below:
                    return True
                else:
                    return False
        else:
            return self.isGreaterThan(other)
    def __ge__(self, other):
        return (self.__eq__(other) or self.__gt__(other))
    def __le__(self, other):
        return (self.__eq__(other) or self.__lt__(other))
    def isLessThan(self, val):
        """ Returns whether the cutpoint is less than a specified value

        Parameters
        ----------
        val : Comparable, of compatible type
            The value to compare the cutpoint to

        Raises
        ------
        ValueError
            If the value type not compatible with cutpoint type

        Returns
        -------
        True if the cutpoint is strictly less than the specified value
        """
        self._validate_query_pt(val)
        if self.belowAll: return True
        elif self.aboveAll:
            return False
        elif self.point < val:
            return True
        elif self.point == val and self.below:
            return True
        else:
            return False
    def isGreaterThan(self, val):
        """ Returns whether the cutpoint is greater than a specified value

        Parameters
        ----------
        val : Comparable, of compatible type
            The value to compare the cutpoint to

        Raises
        ------
        ValueError
            If the value type not compatible with cutpoint type

        Returns
        -------
        True if the cutpoint is strictly greater than the specified value
        """
        self._validate_query_pt(val)
        if self.aboveAll: return True
        elif self.belowAll: return False
        elif self.point > val:
            return True
        elif self.point == val and not self.below:
            return True
        else:
            return False
    @staticmethod
    def belowValue(val, theType = None):
        """ Create a cut point, where everything below some value is
        included

        Parameters
        ----------
        val : value in the domain
            Cut point, where everything below value is included
        theType : type
            Most inclusive type that can be used for comparison. If
            None, then the type of the value is used
        Returns
        -------
        The cut object
        """
        if theType is None:
            return Cut(type(val), point=val, below=True)
        else:
            return Cut(theType, point=val, below=True)
    @staticmethod
    def belowAll(theType):
        """ Create a cut point outside the lower end of the domain

        Parameters
        ----------
        theType : type
            Most inclusive type that can be used for comparison.
         
        Returns
        -------
        The cut object
        """
        return Cut(theType, belowAll=True)
    @staticmethod
    def aboveValue(val, theType=None):
        """ Create a cut point, where everything above some value is
        included

        Parameters
        ----------
        val : value in the domain
            Cut point, where everything above value is included
        theType : type
            Most inclusive type that can be used for comparison. If
            None, then the type of the value is used
         
        Returns
        -------
        The cut object
        """
        if theType is None:
            return Cut(type(val), point=val, below=False)
        else:
            return Cut(theType, point=val, below=False)
    @staticmethod
    def aboveAll(theType):
        """ Create a cut point outside the upper end of the domain

        Parameters
        ----------
        theType : type
            Most inclusive type that can be used for comparison
        
        Returns
        -------
        The cut object
        """
        return Cut(theType, aboveAll=True)

    



from Ranger.src.Range.Cut import Cut

class Range(object):
    """
    Class used to represent a range along some 1-D domain. The range
    is represented by 2 cutpoints can can be unbounded by specifying an
    aboveAll or belowAll Cut.
    """
    def __init__(self, lowerCut, upperCut):
        """ Instantiates a Range

        Parameters
        ----------
        lowerCut : Cut object
            Specifies the lower cut for the range
        upperCut : Cut object
            Specifies the upper cut for the range

        Raises
        ------
        ValueError
            If bound(s) are not Cut objects or lower > upper
        """
        if not all(map(lambda x: isinstance(x, Cut), (lowerCut,upperCut))):
            raise ValueError("Bounds must be Cut objects")
        elif lowerCut > upperCut:
            raise ValueError("Lower bound cannot be greater than upper bound")
        self.lowerCut = lowerCut
        self.upperCut = upperCut
    def __repr__(self):
        try:
            return_str = '[' if self.isLowerBoundClosed() else '('
        except TypeError:
            return_str = '('
        return_str += (str(self.lowerCut.point) if not self.lowerCut.belowAll \
          else '')
        return_str += ' , '
        return_str += (str(self.upperCut.point) if not self.upperCut.aboveAll \
          else '')
        try:
            return_str += ']' if self.isUpperBoundClosed() else ')'
        except TypeError:
            return_str += ')'
        return return_str
    def __hash__(self):
        return (hash(self.lowerCut)*31 + hash(self.upperCut))
    def __eq__(self, other):
        if not isinstance(other, Range):
            return False
        else:
            return ((self.lowerCut == other.lowerCut) and \
                    (self.upperCut == other.upperCut))
    def __ne__(self, other):
        return not self.__eq__(other)
    def contains(self, val):
        """ Returns true if the range contains the value

        Parameters
        ----------
        val : Comparable object of the appropriate type for the range
            Value to query whether in the range

        Raises
        ------
        ValueError
            If the value type not compatible with cutpoint type

        Returns
        -------
        True if the range contains the value
        """
        return (self.lowerCut < val and \
                self.upperCut > val)
    def containsAll(self, vals):
        """ Returns True if the range contains all values in some
        iterable

        Parameters
        ----------
        vals : Iterable of comparable object of appropriate type for range
            Values to query against the range

        Raises
        ------
        ValueError
            If there is a value type not compatible with the cutpoint type

        Returns
        -------
        True if the range contains all values
        """
        for val in vals:
            if not self.contains(val):
                return False
        return True
    def getDistanceFromPoint(self, val, distFunc = lambda x1, x2: abs(x1-x2)):
        """ Returns the minimum distance of a Range from a Point, returning 0
        if there is an overlap.

        Note that both upper and lower bounds must be closed for this function
        to work

        Parameters
        ----------
        val : comparable, compatible with cutpoint type
            The value of the point where the distance is desired
        distFunc : callable
            Function that calculates the distance between two points in the
            domain of the Range

        Raises
        ------
        TypeError
            If the upper and/or lower bounds of this Range are not closed
            or if the distFunc not compatible with the type
        
        Returns
        -------
        The minimum distance between the Range and the Point. Returns 0
        if there is an overlap
        """
        if not all((self.isLowerBoundClosed(), self.isUpperBoundClosed())):
            raise TypeError("Range is not closed")
        if self.contains(val):
            return 0.
        else:
            return min(distFunc(self.lowerCut.point, val),
                       distFunc(self.upperCut.point, val))
    def getDistanceFromRange(self, other, distFunc = lambda x1,x2: abs(x1-x2)):
        """ Returns the minimum distance of a Range from another Range, returning
        0 if there is any overlap

        Note that both Ranges must be closed for this function to work

        Parameters
        ----------
        other : Range, compatible with this Range's domain
            The Range to compare to
        distFunc : callable
            Function that calculates the distance between two points in the
            domain of the Range

        Raises
        ------
        TypeError
            If the upper and/or lower bounds of this Range are not closed
            or if the distFunc not compatible with the type

        Returns
        -------
        Minimum distance between the ranges        
        """
        if not isinstance(other, Range):
            raise TypeError("other is not a Range")
        if not all((self.isLowerBoundClosed(), self.isUpperBoundClosed(),
                    other.isLowerBoundClosed(), other.isUpperBoundClosed())):
            raise TypeError("Not all Ranges closed")
        if self.isConnected(other):
            return 0.
        else:
            return min(distFunc(self.lowerCut.point, other.upperCut.point),
                       distFunc(other.lowerCut.point, self.upperCut.point))
    def hasLowerBound(self):
        """ Returns True if the range has a lower endpoint (not unbounded
        at the lower end)

        Returns
        -------
        True if the range has a lower endpoint
        """
        return (not self.lowerCut.belowAll)
    def hasUpperBound(self):
        """ Returns True if the range has an upper endpoint (not unbounded
        at the upper end)

        Returns
        -------
        True if the range has an upper endpoint
        """
        return (not self.upperCut.aboveAll)
    def lowerEndpoint(self):
        """ Returns the lower endpoint of the range if it exists. Otherwise
        raises a TypeError

        Raises
        ------
        TypeError
            If the range is unbounded below

        Returns
        -------
        The lower endpoint of the range
        """
        if self.lowerCut.point is None:
            raise TypeError("Range unbounded below")
        else:
            return self.lowerCut.point
    def upperEndpoint(self):
        """ Returns the upper endpoint of the range if it exists. Otherwise
        raises a TypeError

        Raises
        ------
        TypeError
            If the range is unbounded above

        Returns
        -------
        The upper endpoint of the range
        """
        if self.upperCut.point is None:
            raise TypeError("Range unbounded above")
        else:
            return self.upperCut.point
    def isLowerBoundClosed(self):
        """ Returns whether the lower bound is closed (if there is a
        lower bound)

        Raises
        ------
        TypeError
            If the range is unbounded below

        Returns
        -------
        True if the lower bound is closed
        """
        if self.lowerCut.point is None:
            raise TypeError("Range unbounded below")
        else:
            return self.lowerCut.below
    def isUpperBoundClosed(self):
        """ Returns whether the upper bound is closed (if there is an
        upper bound)

        Raises
        ------
        TypeError
            If the range is unbounded above

        Returns
        -------
        True if the upper bound is closed
        """
        if self.upperCut.point is None:
            raise TypeError("Range unbounded above")
        else:
            return (not self.upperCut.below)
    def isEmpty(self):
        """ Returns True if the range is of form [v, v) or (v, v]

        Returns
        -------

        True if the range is of the form [v,v) or (v,v]
        """
        return self.lowerCut == self.upperCut
    def encloses(self, other):
        """ Returns True if the bounds of the other range do not extend
        outside the bounds of this range

        Examples:
            [3,6] encloses [4,5]
            (3,6) encloses (3,6)
            [3,6] encloses [4,4]
            (3,6] does not enclose [3,6]
            [4,5] does not enclose (3,6)

        Parameters
        ----------
        other : A Range
            The range to compare to

        Raises
        ------
        ValueError
            If object passed in is not a Range

        Returns
        -------
        True if the bounds of the other range do not extend outside
        the bounds of this range
        """
        if not isinstance(other, Range):
            raise ValueError("Range required")
        return ((self.lowerCut <= other.lowerCut) and \
            (self.upperCut >= other.upperCut))
    def isConnected(self, other):
        """ Returns True if there is a (possibly empty) range that is
        enclosed by both this range and other

        Examples:
            [2,4] and [5,7] are not connected
            [2,4] and [3,5] are connected
            [2,4] and [4,6] are connected
            [3,5] and (5,10) are connected
        
        Parameters
        ----------
        other : A range
            The range to compare to

        Raises
        ------
        ValueError
            If object passed in is not a Range
        
        Returns
        -------
        True if there is a (possibly empty) range that is enclosed by
        both this range and other
        """
        if not isinstance(other, Range):
            raise ValueError("Range required")
        return ((self.lowerCut <= other.upperCut) and \
                (other.lowerCut <= self.upperCut))
    def intersection(self, other):
        """ Returns the maximal range enclosed by both this range and the
        other range, if such a range exists

        Examples:
            Intersection of [1,5] and [3,7] is [3,5]
            Intersection of [1,5] and [5,7] is [5,5]

        Parameters
        ----------
        other : A range
            The range to compare to

        Raises
        ------
        ValueError
            If object passed in is not a Range or if there is no intersection

        Returns
        -------
        The intersection range
        """
        if not isinstance(other, Range):
            raise ValueError("Range required")
        if ((self.lowerCut >= other.lowerCut) and \
            (self.upperCut <= other.upperCut)):
            return Range(self.lowerCut, self.upperCut)
        elif ((self.lowerCut <= other.lowerCut) and \
              (self.upperCut >= other.upperCut)):
            return Range(other.lowerCut, other.upperCut)
        else:
            newLower = self.lowerCut if (self.lowerCut >= other.lowerCut) else \
                                         other.lowerCut
            newUpper = self.upperCut if (self.upperCut <= other.upperCut) else \
                                         other.upperCut
            return Range(newLower, newUpper)
    def span(self, other):
        """ Returns the minimal range that encloses both this range and
        the other. Note that if the input ranges are not connected, the span can
        contain values that are not contained within either input range

        Examples:
            Span of [1,3] and [5,7] is [1,7]

        Parameters
        ----------
        other : A range
            A range to span with

        Raises
        ------
        ValueError
            If object passed in is not a Range or if there is no intersection
        
        Returns
        -------
        The minimal range enclosing both with and the other range
        """
        if ((self.lowerCut <= other.lowerCut) and \
            (self.upperCut >= other.upperCut)):
            return Range(self.lowerCut, self.upperCut)
        elif ((self.lowerCut >= other.lowerCut) and \
              (self.upperCut <= other.upperCut)):
            return Range(other.lowerCut, other.upperCut)
        else:
            newLower = self.lowerCut if (self.lowerCut <= other.lowerCut) else \
                other.lowerCut
            newUpper = self.upperCut if (self.upperCut >= other.upperCut) else \
                other.upperCut
            return Range(newLower, newUpper)
    ##################
    # Static methods #
    ##################
    @staticmethod
    def _validate_cutpoints(*pts):
        if not all(map(lambda x: (hasattr(x, "__lt__") and \
                hasattr(x, "__gt__")) or hasattr(x,'__cmp__'), pts)):
            raise ValueError("Cutpoint type(s) not comparable")
        if len(pts) == 2:
            if not (issubclass(type(pts[0]),type(pts[1])) or \
              issubclass(type(pts[1]),type(pts[0]))):
                raise ValueError("Cutpoints are not compatible")
        return True
    @staticmethod
    def _get_type(*pts):
        if len(pts) == 1: return type(pts[0])
        elif len(pts) == 2:
            if issubclass(type(pts[0]),type(pts[1])):
                return type(pts[1])
            elif issubclass(type(pts[1]),type(pts[0])):
                return type(pts[0])
            else:
                raise ValueError("Cutpoints are not compatible")
    @staticmethod
    def closed(lower, upper):
        """ Creates a range including the endpoints (i.e. [lower, upper])

        Parameters
        ----------
        lower : comparable, of same type as or subclass of upper type
            The lower bound
        upper : comparable, of same type as or subclass of lower type
            The upper bound

        Raises
        ------
        ValueError
            If type(s) are not comparable or compatible
        
        Returns
        -------
        A Range object [lower, upper]
        """
        # Ensure cutpoints are of compatible, appropriate types
        Range._validate_cutpoints(lower, upper)
        theType = Range._get_type(lower,upper)
        return Range(Cut.belowValue(lower, theType=theType),
                     Cut.aboveValue(upper, theType=theType))
    @staticmethod
    def closedOpen(lower, upper):
        """ Creates a range including the lower endpoint (i.e. [lower, upper))

        Parameters
        ----------
        lower : comparable, of same type as or subclass of upper type
            The lower bound
        upper : comparable, of same type as or subclass of lower type
            The upper bound

        Raises
        ------
        ValueError
            If type(s) are not comparable or compatible
        
        Returns
        -------
        A Range object [lower, upper)
        """
        # Ensure cutpoints are of compatible, appropriate types
        Range._validate_cutpoints(lower, upper)
        theType = Range._get_type(lower,upper)
        return Range(Cut.belowValue(lower, theType=theType),
                     Cut.belowValue(upper, theType=theType))
    @staticmethod
    def openClosed(lower, upper):
        """ Creates a range including the upper (i.e. (lower, upper])

        Parameters
        ----------
        lower : comparable, of same type as or subclass of upper type
            The lower bound
        upper : comparable, of same type as or subclass of lower type
            The upper bound

        Raises
        ------
        ValueError
            If type(s) are not comparable or compatible
        
        Returns
        -------
        A Range object (lower, upper]
        """
        # Ensure cutpoints are of compatible, appropriate types
        Range._validate_cutpoints(lower, upper)
        theType = Range._get_type(lower,upper)
        return Range(Cut.aboveValue(lower, theType=theType),
                     Cut.aboveValue(upper, theType=theType))
    @staticmethod
    def open(lower, upper):
        """ Creates a range excluding the endpoints (i.e. (lower, upper))

        Parameters
        ----------
        lower : comparable, of same type as or subclass of upper type
            The lower bound
        upper : comparable, of same type as or subclass of lower type
            The upper bound

        Raises
        ------
        ValueError
            If type(s) are not comparable or compatible or if constructing
            a range of type (v,v), which is invalid
        
        Returns
        -------
        A Range object (lower, upper)
        """
        # Ensure cutpoints are of compatible, appropriate types
        Range._validate_cutpoints(lower, upper)
        theType = Range._get_type(lower,upper)
        if lower == upper:
            raise TypeError("Range of type (v,v) is not valid")        
        return Range(Cut.aboveValue(lower, theType=theType),
                     Cut.belowValue(upper, theType=theType))
    @staticmethod
    def lessThan(val):
        """ Makes range including all values less than some value
        (i.e. (-inf, val))

        Parameters
        ----------
        val : comparable
            The upper bound

        Raises
        ------
        ValueError
            If type not comparable

        Returns
        -------
        A Range object (-inf, val)
        """
        Range._validate_cutpoints(val)
        theType = Range._get_type(val)
        return Range(Cut.belowAll(theType=theType),
                     Cut.belowValue(val, theType=theType))
    @staticmethod
    def atMost(val):
        """ Makes range including all values less than or equal to
        some value (i.e. (-inf, val])

        Parameters
        ----------
        val : comparable
            The upper bound

        Raises
        ------
        ValueError
            If type not comparable

        Returns
        -------
        A Range object (-inf, val]
        """
        Range._validate_cutpoints(val)
        theType = Range._get_type(val)
        return Range(Cut.belowAll(theType=theType),
                     Cut.aboveValue(val, theType=theType))
    @staticmethod
    def greaterThan(val):
        """ Makes range including all values greater than
        some value (i.e. (val, inf])

        Parameters
        ----------
        val : comparable
            The lower bound

        Raises
        ------
        ValueError
            If type not comparable

        Returns
        -------
        A Range object (val, inf)
        """
        Range._validate_cutpoints(val)
        theType = Range._get_type(val)
        return Range(Cut.aboveValue(val,theType=theType),
                     Cut.aboveAll(theType=theType))
    @staticmethod
    def atLeast(val):
        """ Makes range including all values greater than or equal to
        some value (i.e. [val, inf))

        Parameters
        ----------
        val : comparable
            The lower bound

        Raises
        ------
        ValueError
            If type not comparable

        Returns
        -------
        A Range object [val, inf)
        """
        Range._validate_cutpoints(val)
        theType = Range._get_type(val)
        return Range(Cut.belowValue(val, theType=theType),
                     Cut.aboveAll(theType=theType))



from bisect import bisect_left
from Ranger.src.Range.Range import Range
from collections import deque

class RangeMap(object):
    """ Class used to represent a mapping of disjoint ranges to some objects.
    Ranges do not coalesce. If a new Range is added over an existing Range,
    it overwrites the overlapping part of the existing Range
    """
    def __init__(self, rangeDict = None):
        """ Instantiates a RangeMap
        
        Parameters
        ----------
        rangeDict : Dictionary of Range -> object
            Dictionary to start off the RangeMap with. Note that this will
            not be traversed in any particular order, so it may result in
            unexpected behavior if instantiated with any overlapping ranges
        """
        # Holds lower and upper cut points of ranges
        self.lower_cuts = []
        self.upper_cuts = []
        # Holds the actual range objects that are the keys
        self.ranges = []
        # Holds items mapping to each range
        self.items = []
        if rangeDict is not None:
            for rangeKey, val in rangeDict.iteritems():
                self.put(rangeKey, val)
    def __getitem__(self, key):
        return self.get(key)
    def __setitem__(self, key, value):
        self.put(key, value)
    def __delitem__(self, key):
        self.remove(key)
    def __iter__(self):
        return iter(self.ranges)
    def __eq__(self, other):
        if not isinstance(other, RangeMap): return False
        elif len(self) != len(other): return False
        for k1,v1,k2,v2 in zip(self.ranges, self.items,
                               other.ranges, other.items):
            if (k1 != k2 or v1 != v2):
                return False
        return True
    def __ne__(self, other):
        return not self.__eq__(other)
    def __len__(self):
        return len(self.ranges)
    def __repr__(self):
        if len(self) < 5:
            returnStr = "{%s}" % ", ".join([
                "%s : %s" % (k,v) for k,v in zip(self.ranges, self.items)
                ])
        else:
            returnStr = "{%s, ..., %s}" % (", ".join([
                "%s : %s" % (k,v) for k,v in zip(self.ranges[:2], self.items[:2])
            ]), ", ".join([
                "%s : %s" % (k,v) for k,v in zip(self.ranges[-2:], self.items[-2:])
            ])
                                           )
        return returnStr
    def __missing__(self, key):
        raise KeyError(str(key))
    def contains(self, val):
        """ Returns true if any of the ranges fully enclose the given
        value, which can be a single value or a Range object

        Parameters
        ----------
        val : A single value or a Range object

        Raises
        ------
        ValueError
            If the value type not compatible with the ranges
        
        Returns
        -------
        true if any of the ranges fully enclose the given value
        """
        if len(self) == 0: return False
        # Get the index+1 of the highest lower cut <= to the value or its
        # lower cutpoint and check if the value contained
        if isinstance(val, Range):
            lower_ind = max(bisect_left(self.lower_cuts, val.lowerCut)-1,0)
            return self.ranges[lower_ind].encloses(val)
        else:
            lower_ind = max(bisect_left(self.lower_cuts, val)-1,0)
            return self.ranges[lower_ind].contains(val)
    def get(self, key):
        """ Get the item(s) corresponding to a given key. The key can be a
        Range or a single value that is within a Range

        Parameters
        ----------
        key : A single value or Range object

        Raises
        ------
        KeyError
            If there is no overlap with the key
        ValueError
            If the key type not compatible with the ranges
        
        Returns
        -------
        A set containing all overlapping items
        """
        if not self.overlaps(key):
            self.__missing__(key)
        elif isinstance(key, Range):
            # If this is a single value
            returnSet = set()
            # Get the bounding indices
            ovlapLowerInd = max(bisect_left(self.lower_cuts, key.lowerCut)-1,0)
            ovlapUpperInd = bisect_left(self.lower_cuts, key.upperCut)
            for i in range(ovlapLowerInd, ovlapUpperInd):
                try:
                    # Get intersection of the ranges
                    intersect = key.intersection(self.ranges[i])
                    if not intersect.isEmpty():
                        # If overlapping with this range, put its
                        # item in the return set
                        returnSet.add(self.items[i])
                except ValueError:
                    # Continue if no overlap with this range
                    continue
            # Return the set of items
            return returnSet
        else:
            # If this is a single value
            # Get the index of the range containing the value
            lower_ind = max(bisect_left(self.lower_cuts, key)-1,0)
            # Return the item at that value
            return set([self.items[lower_ind]])
            
    def overlaps(self, val):
        """ Returns true if any of the ranges at least partially overlap
        the given value, which can be a single value or a Range object

        Parameters
        ----------
        val : A single value or a Range object

        Raises:
        -------
        ValueError
            If the value type not compatible with the ranges

        Returns
        -------
        true if any of the ranges fully enclose the given value
        """
        if len(self) == 0: return False
        # Get the index+1 of the highest lower cut <= to the value or its
        # lower cutpoint and check if the value overlaps
        if isinstance(val, Range):
            lower_ind = bisect_left(self.lower_cuts, val.lowerCut)-1
            upper_ind = bisect_left(self.lower_cuts, val.upperCut)
            for i in range(lower_ind,upper_ind):
                if val.isConnected(self.ranges[i]):
                    if not self.ranges[i].intersection(val).isEmpty():
                        return True
            return False
        else:
            lower_ind = bisect_left(self.lower_cuts,val)-1
            return self.ranges[lower_ind].contains(val)        
    def put(self, key, val):
        """ Creates a mapping from a Range to a value. Note that if the
        key Range overlaps any existing ranges, it will replace those
        Range(s) over the intersection

        Parameters
        ----------
        key : Range object
            A Range to serve as a key
        val : value
            Some value that the Range should map to

        Raises
        ------
        TypeError
            If the key is not a Range object
        """
        if not isinstance(key, Range):
            raise TypeError("key is not a Range")
        elif key.isEmpty():
            # Skip if this is an empty range
            return
        # Figure out where to the key/value
        if not self.overlaps(key):
            # If this range is completely on its own, just insert
            insertInd = bisect_left(self.lower_cuts, key.lowerCut)
            self.ranges.insert(insertInd, key)
            self.lower_cuts.insert(insertInd, key.lowerCut)
            self.upper_cuts.insert(insertInd, key.upperCut)
            self.items.insert(insertInd, val)
            return
        else:
            # If this range has some overlap with existing ranges
            ovlapLowerInd = max(bisect_left(self.lower_cuts, key.lowerCut)-1,0)
            ovlapUpperInd = bisect_left(self.lower_cuts, key.upperCut)
            # Create queue or indices marked for removal
            removeRanges = deque()
            # Create queue ranges to add
            addRanges = deque()
            # Create queue of items to add
            addItems = deque()
            for i in range(ovlapLowerInd, ovlapUpperInd):
                try:
                    # Get intersection of the ranges
                    intersect = key.intersection(self.ranges[i])
                    if not intersect.isEmpty():
                        if intersect == self.ranges[i]:
                            # Mark range for removal
                            removeRanges.append(i)
                        elif self.lower_cuts[i] == intersect.lowerCut:
                            # If equal on left cutpoint, subtract out left
                            # part
                            self.lower_cuts[i] = intersect.upperCut
                            self.ranges[i] = Range(intersect.upperCut,
                                                   self.upper_cuts[i])
                        elif self.upper_cuts[i] == intersect.upperCut:
                            # If equal on right cutpoint, subtract out
                            # right part
                            self.upper_cuts[i] = intersect.lowerCut
                            self.ranges[i] = Range(self.lower_cuts[i],
                                                   intersect.lowerCut)
                        else:
                            # If in the middle, split into two parts, putting
                            # both in add queue and placing the old range index
                            # in the remove queue
                            addRanges.append(Range(self.lower_cuts[i],
                                                   intersect.lowerCut))
                            addRanges.append(Range(intersect.upperCut,
                                                   self.upper_cuts[i]))
                            addItems.append(self.items[i])
                            addItems.append(self.items[i])
                            removeRanges.append(i)
                except ValueError:
                    # Continue if no overlap with this range
                    continue
            # Remove any ranges that are marked for removal
            while len(removeRanges) > 0:
                removeInd = removeRanges.pop()
                self.ranges.pop(removeInd)
                self.lower_cuts.pop(removeInd)
                self.upper_cuts.pop(removeInd)
                self.items.pop(removeInd)
            addItems.append(val)
            addRanges.append(key)
            # Use recursive call to place the pairs, which now
            # should not overlap with any other ranges
            while len(addRanges) > 0:
                self.put(addRanges.pop(),addItems.pop())
    def remove(self, aRange):
        """ Removes a range and its value from the range set

        Parameters
        ----------
        aRange : A Range object
            The Range to remove

        Raises
        ------
        ValueError
            If removing range of type not compatible with previously
            added ranges
        TypeError
            If not a Range
        """
        if not isinstance(aRange, Range):
            raise TypeError("aRange is not a Range")
        elif aRange.isEmpty():
            # Skip if this is an empty range
            return
        # Check for compatibility of types if necessary
        if len(self) > 0:
            if not (issubclass(aRange.lowerCut.theType,
                               self.ranges[0].lowerCut.theType) or \
                    issubclass(self.ranges[0].lowerCut.theType,
                               aRange.lowerCut.theType)):
                raise ValueError("Range not compatible with previously added ranges")
        # Check if the range actually overlaps with the key set
        if not self.overlaps(aRange):
            return
        else:
            # There's some overlap, so deal with that
            # Determine where overlap occurs
            ovlapLowerInd = max(bisect_left(self.lower_cuts,
                                            aRange.lowerCut)-1,0)
            ovlapUpperInd = bisect_left(self.lower_cuts, aRange.upperCut)
            # Create queue of indices marked for removal
            removeRanges = deque()
            # Create queue of ranges to add
            addRanges = deque()
            # Create queue of items to add with the addRanges
            addItems = deque()
            for i in range(ovlapLowerInd, ovlapUpperInd):
                try:
                    # Get intersection of the ranges
                    intersect = aRange.intersection(self.ranges[i])
                    if not intersect.isEmpty():
                        if intersect == self.ranges[i]:
                            # Mark range for removal
                            removeRanges.append(i)
                        elif self.lower_cuts[i] == intersect.lowerCut:
                            # If equal on the left cutpoint, subtract
                            # out left part
                            self.lower_cuts[i] = intersect.upperCut
                            self.ranges[i] = Range(intersect.upperCut,
                                                   self.upper_cuts[i])
                        elif self.upper_cuts[i] == intersect.upperCut:
                            # If equal on right cutpoint, subtract out
                            # right part
                            self.upper_cuts[i] = intersect.lowerCut
                            self.ranges[i] = Range(self.lower_cuts[i],
                                                   intersect.lowerCut)
                        else:
                            # If in the middle, split into two parts, putting
                            # both in add queue and placing the old range index
                            # into the remove queue
                            addRanges.append(Range(self.lower_cuts[i],
                                                   intersect.lowerCut))
                            addItems.append(self.items[i])
                            addRanges.append(Range(intersect.upperCut,
                                                   self.upper_cuts[i]))
                            addItems.append(self.items[i])
                            removeRanges.append(i)
                except ValueError:
                    # Continue if no overlap with this range
                    continue
            # Remove any ranges that are marked for removal
            while len(removeRanges) > 0:
                removeInd = removeRanges.pop()
                self.ranges.pop(removeInd)
                self.lower_cuts.pop(removeInd)
                self.upper_cuts.pop(removeInd)
                self.items.pop(removeInd)
            # Add any pairs that need to be added
            while len(addRanges) > 0:
                self.put(addRanges.pop(), addItems.pop())



import unittest
from Ranger.test.src.Collections.RangeSetTest import RangeSetTest
from Ranger.test.src.Collections.RangeMapTest import RangeMapTest

class CollectionsTestSuite(unittest.TestSuite):
    def __init__(self):
        super(CollectionsTestSuite, self).__init__()
        self.addTest(unittest.makeSuite(RangeSetTest))
        self.addTest(unittest.makeSuite(RangeMapTest))

if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(CollectionsTestSuite())



import unittest
from Ranger.src.Collections.RangeSet import RangeSet
from Ranger.src.Range.Range import Range
from Ranger.src.Range.Cut import Cut

debug = False

class RangeSetTest(unittest.TestCase):
    """ Unit Tests for RangeSet.py """
    def test_add(self):
        if debug: print("Testing add with integers")
        theSet = RangeSet()
        # Adding initial part
        theSet.add(Range.closed(3,5))
        self.assertEqual(theSet.lower_cuts[0], Cut.belowValue(3))
        self.assertEqual(theSet.upper_cuts[0], Cut.aboveValue(5))
        self.assertEqual(len(theSet),1)
        # Adding distinct range above initial one
        theSet.add(Range.closed(7,10))
        self.assertEqual(len(theSet),2)
        self.assertEqual(theSet.ranges[1],Range.closed(7,10))
        self.assertEqual(theSet.ranges[0],Range.closed(3,5))
        # Adding range below/overlapping with initial one
        theSet.add(Range.closed(2,3))
        self.assertEqual(len(theSet),2)
        self.assertEqual(Range.closed(2,5), theSet.ranges[0])
        self.assertEqual(Cut.belowValue(2), theSet.lower_cuts[0])
        self.assertEqual(Cut.aboveValue(5), theSet.upper_cuts[0])
        self.assertEqual(Range.closed(7,10), theSet.ranges[1])
        self.assertEqual(Cut.belowValue(7), theSet.lower_cuts[1])
        self.assertEqual(Cut.aboveValue(10), theSet.upper_cuts[1])
        # Adding range above/overlapping second one
        theSet.add(Range.closed(9,11))
        self.assertEqual(len(theSet),2)
        self.assertEqual(Range.closed(7,11), theSet.ranges[1])
        self.assertEqual(Cut.belowValue(7), theSet.lower_cuts[1])
        self.assertEqual(Cut.aboveValue(11), theSet.upper_cuts[1])
        # Adding range encompasing second one
        theSet.add(Range.closed(6,12))
        self.assertEqual(len(theSet),2)
        self.assertEqual(Range.closed(6,12), theSet.ranges[1])
        self.assertEqual(Cut.belowValue(6), theSet.lower_cuts[1])
        self.assertEqual(Cut.aboveValue(12), theSet.upper_cuts[1])
        # Adding range encompassing all
        theSet.add(Range.closed(3, 11))
        self.assertEqual(len(theSet),1)
        self.assertEqual(len(theSet.lower_cuts),1)
        self.assertEqual(len(theSet.upper_cuts),1)
        self.assertEqual(Range.closed(2,12), theSet.ranges[0])
        self.assertEqual(Cut.belowValue(2), theSet.lower_cuts[0])
        self.assertEqual(Cut.aboveValue(12), theSet.upper_cuts[0])
    def test_contains(self):
        if debug: print("Testing contains")
        theSet = RangeSet()
        theSet.add(Range.closed(3,5))
        theSet.add(Range.closed(7,10))
        self.assertTrue(theSet.contains(4))
        self.assertFalse(theSet.contains(2))
        self.assertTrue(theSet.contains(Range.closed(4,5)))
        self.assertTrue(theSet.contains(Range.closed(8,9)))
        self.assertFalse(theSet.contains(Range.closed(1,4)))
        self.assertFalse(theSet.contains(Range.closed(6,6)))
        self.assertFalse(theSet.contains(Range.closed(8,12)))
    def test_overlaps(self):
        if debug: print("Testing overlaps")
        theSet = RangeSet()
        theSet.add(Range.closed(3,5))
        theSet.add(Range.closed(7,10))
        self.assertTrue(theSet.overlaps(4))
        self.assertFalse(theSet.overlaps(Range.closedOpen(1,3)))
        self.assertFalse(theSet.overlaps(Range.open(5,7)))
        self.assertFalse(theSet.overlaps(2))
        self.assertTrue(theSet.overlaps(3))
        self.assertTrue(theSet.overlaps(10))
        self.assertTrue(theSet.overlaps(Range.closed(4,5)))
        self.assertTrue(theSet.overlaps(Range.closed(8,9)))
        self.assertTrue(theSet.overlaps(Range.closed(1,4)))
        self.assertFalse(theSet.overlaps(Range.closed(6,6)))
        self.assertTrue(theSet.overlaps(Range.closed(8,12)))
        self.assertTrue(theSet.overlaps(Range.closed(1,12)))
    def test_union(self):
        if debug: print("Testing union")
        firstSet = RangeSet([Range.closed(3,5), Range.closed(7,10)])
        secondSet = RangeSet([Range.closed(2,4), Range.closed(5, 11),
                              Range.closed(13, 15)])
        union = firstSet.union(secondSet)
        self.assertEqual(union, RangeSet([Range.closed(2,11),
                                          Range.closed(13,15)]))
    def test_difference(self):
        if debug: print("Testing difference")
        startSet = RangeSet([Range.closed(3,5),Range.closed(7,10)])
        diffSet = startSet.difference(RangeSet([Range.closed(4,6)]))
        self.assertEqual(diffSet,
                         RangeSet([Range.closedOpen(3,4),
                                   Range.closed(7,10)]))
        diffSet = startSet.difference(RangeSet([Range.closed(2,6)]))
        self.assertEqual(diffSet,
                         RangeSet([Range.closed(7,10)]))
        diffSet = startSet.difference(RangeSet([Range.closed(-2,1)]))
        self.assertEqual(diffSet, startSet)
        diffSet = startSet.difference(RangeSet([Range.closed(1,3),
                                                Range.closed(6,9)]))
        self.assertEqual(diffSet,
                         RangeSet([Range.openClosed(3,5),
                                   Range.openClosed(9,10)]))
        diffSet = startSet.difference(RangeSet([Range.closed(1,11)]))
        self.assertEqual(len(diffSet),0)
    def test_difference_bug1(self):
        if debug: print("Testing difference with reference to bug 1")
        set1 = RangeSet([Range.closedOpen(2949490, 2949550),
                         Range.closedOpen(2950080, 2950130),
                         Range.closedOpen(2993110, 2993150)])
        set2 = RangeSet([
            Range.closedOpen(2949640, 2949660),
            Range.closedOpen(2950070, 2950090),
            Range.closedOpen(2950110, 2950120)
        ])
        diff = set1.difference(set2)
        self.assertEqual(diff, RangeSet([
            Range.closedOpen(2949490, 2949550),
            Range.closedOpen(2950090, 2950110),
            Range.closedOpen(2950120, 2950130),
            Range.closedOpen(2993110, 2993150)
        ]))
                                        
    def test_intersection(self):
        if debug: print("Testing intersection")
        startSet = RangeSet([Range.closed(3,5),Range.closed(7,10)])
        intersect = startSet.intersection(RangeSet([Range.closed(4,6)]))
        self.assertEqual(intersect,RangeSet([Range.closed(4,5)]))
        intersect = startSet.intersection(RangeSet([
            Range.closed(1,3),Range.closed(8,9),Range.closed(12,15)
        ]))
        self.assertEqual(intersect, RangeSet([
            Range.closed(3,3),Range.closed(8,9)
        ]))
        intersect = startSet.intersection(RangeSet([
            Range.openClosed(8,1000)
        ]))
        self.assertEqual(intersect, RangeSet([
            Range.openClosed(8,10)
        ]))
    def test_remove(self):
        if debug: print("Testing remove")
        startSet = RangeSet([Range.closed(3,5),Range.closed(7,10)])
        # Try removing something outside all ranges
        startSet.remove(Range.closed(0,1))
        self.assertEqual(startSet, RangeSet([
            Range.closed(3,5),Range.closed(7,10)
        ]))
        # Remove left part of left range
        startSet.remove(Range.closed(2,3))
        self.assertEqual(startSet, RangeSet([
            Range.openClosed(3,5),Range.closed(7,10)
        ]))
        # Remove from middle of right range
        startSet.remove(Range.closed(8,9))
        self.assertEqual(startSet, RangeSet([
            Range.openClosed(3,5),Range.closedOpen(7,8),
            Range.openClosed(9,10)
        ]))
        # Remove through both ranges
        startSet.remove(Range.closed(5,7))
        self.assertEqual(startSet, RangeSet([
            Range.open(3,5),Range.open(7,8),
            Range.openClosed(9,10)
        ]))
        # Remove over the whole second set of ranges
        startSet.remove(Range.closed(5,100))
        self.assertEqual(startSet, RangeSet([
            Range.open(3,5)
        ]))
if __name__ == "__main__":
    debug = True
    unittest.main(exit = False)



import unittest
from Ranger.src.Range.Cut import Cut

debug = False

class CutTest(unittest.TestCase):
    """ Unit Tests for Cut.py """
    # TODO: Write unit tests for __eq__, __lt__, __gt__, __le__, __ge__
    def test_isLessThanInt(self):
        if debug: print("Testing isLessThan with integers")
        ptCut = Cut(int, point=2, below=False)
        belowAllCut = Cut(int, belowAll=True)
        aboveAllCut = Cut(int, aboveAll = True)
        self.assertTrue(ptCut.isLessThan(3))
        self.assertFalse(ptCut.isLessThan(2))
        self.assertFalse(ptCut.isLessThan(1))
        self.assertTrue(belowAllCut.isLessThan(-999))
        self.assertFalse(aboveAllCut.isLessThan(1000))
    def test_isGreaterThanInt(self):
        if debug: print("Testing isGreaterThan with integers")
        ptCut = Cut(int, point=2, below=False)
        belowAllCut = Cut(int, belowAll=True)
        aboveAllCut = Cut(int, aboveAll = True)
        self.assertFalse(ptCut.isGreaterThan(3))
        self.assertTrue(ptCut.isGreaterThan(2))
        self.assertTrue(ptCut.isGreaterThan(1))
        self.assertFalse(belowAllCut.isGreaterThan(-999))
        self.assertTrue(aboveAllCut.isGreaterThan(1000))
    def test_belowValue(self):
        if debug: print("Testing belowValue")
        theCut = Cut.belowValue(2)
        self.assertFalse(theCut.belowAll)
        self.assertFalse(theCut.aboveAll)
        self.assertEqual(theCut.point, 2)
        self.assertTrue(theCut.below)
    def test_belowAll(self):
        if debug: print("Testing belowAll")
        theCut = Cut.belowAll(int)
        self.assertTrue(theCut.belowAll)
        self.assertFalse(theCut.aboveAll)
        self.assertIsNone(theCut.point)
    def test_aboveValue(self):
        if debug: print("Testing aboveValue")
        theCut = Cut.aboveValue(2)
        self.assertFalse(theCut.belowAll)
        self.assertFalse(theCut.aboveAll)
        self.assertEqual(theCut.point, 2)
        self.assertFalse(theCut.below)
    def test_aboveAll(self):
        if debug: print("Testing aboveAll")
        theCut = Cut.aboveAll(int)
        self.assertFalse(theCut.belowAll)
        self.assertTrue(theCut.aboveAll)
        self.assertIsNone(theCut.point)        
if __name__ == "__main__":
    debug = True
    unittest.main(exit=False)



"""
Version information for Ranger, created during installation.

Do not add this file to the repository.

"""

import datetime

version = '0.1.dev_20140719184002'
date = 'Sat Jul 19 18:40:02 2014'

# Was Ranger built from a development version? If so, remember that the major
# and minor versions reference the "target" (rather than "current") release.
dev = True

# Format: (name, major, min, revision)
version_info = ('Ranger', '0', '1', None)

# Format: a 'datetime.datetime' instance
date_info = datetime.datetime(2014, 7, 19, 18, 40, 2, 314838)

# Format: (vcs, vcs_tuple)
vcs_info = (None, (None, None))




import unittest
from Ranger.src.Collections.RangeBucketMap import RangeBucketMap
from Ranger.src.Range.Range import Range

debug = False

class RangeBucketMapTest(unittest.TestCase):
    """ Unit Tests for RangeBucketMap.py """
    def test_put(self):
        if debug: print("Testing put")
        buckets = RangeBucketMap()
        buckets.put(Range.closed(3,5),'a')
        self.assertEqual(buckets.ranges[0], Range.closed(3,5))
        self.assertEquals(buckets.items[0], set(['a']))
        buckets.put(Range.closed(7,10), 'b')
        self.assertEqual(len(buckets),2)
        self.assertEqual(buckets.ranges[0], Range.closed(3,5))
        self.assertEqual(buckets.ranges[1], Range.closed(7,10))
        self.assertEqual(buckets.items[0], set(['a']))
        self.assertEqual(buckets.items[1], set(['b']))
        buckets.put(Range.closed(4,8),'c')
        self.assertEqual(len(buckets),5)
        self.assertEqual(buckets.ranges[0], Range.closedOpen(3,4))
        self.assertEqual(buckets.ranges[1], Range.closed(4,5))
        self.assertEqual(buckets.ranges[2], Range.open(5,7))
        self.assertEqual(buckets.ranges[3], Range.closed(7,8))
        self.assertEqual(buckets.ranges[4], Range.openClosed(8,10))
        self.assertEqual(buckets.items[0], set(['a']))
        self.assertEqual(buckets.items[1], set(['a','c']))
        self.assertEqual(buckets.items[2], set(['c']))
        self.assertEqual(buckets.items[3], set(['b','c']))
        self.assertEqual(buckets.items[4], set(['b']))
    def test_get(self):
        if debug: print("Testing get")
        buckets = RangeBucketMap()
        buckets.put(Range.closed(3,5),'a')
        buckets.put(Range.closed(7,10),'b')
        buckets.put(Range.closed(4,8),'c')
        self.assertEqual(buckets.get(6),set(['c']))
        self.assertEqual(buckets.get(4),set(['a','c']))
        self.assertEquals(buckets.get(Range.closed(0,20)),set(['a','b','c']))
        self.assertEquals(buckets.get(Range.openClosed(5,8)),set(['b','c']))
    def test_get_bugfix1(self):
        if debug: print("Testing get under first bugfix")
        buckets = RangeBucketMap()
        buckets.put(Range.closed(67432367,67434244),'G')
        buckets.put(Range.closed(67432367,67434244),'T1')
        buckets.put(Range.closed(67432375,67434015),'T2')
        buckets_dict = dict((v,k) for k,v in buckets.iteritems())
        self.assertEqual(buckets_dict['T2'], Range.closed(67432375,67434015))
    def test_remove(self):
        if debug: print("Testing remove")
        buckets = RangeBucketMap()
        buckets.put(Range.closed(3,5),'a')
        buckets.put(Range.closed(7,10),'b')
        buckets.put(Range.closed(4,8),'c')
        buckets.remove(Range.closed(3,3))
        self.assertEqual(len(buckets),5)
        self.assertEqual(buckets.ranges[0], Range.open(3,4))
        self.assertEqual(buckets.ranges[1], Range.closed(4,5))
        self.assertEqual(buckets.ranges[2], Range.open(5,7))
        self.assertEqual(buckets.ranges[3], Range.closed(7,8))
        self.assertEqual(buckets.ranges[4], Range.openClosed(8,10))
        self.assertEqual(buckets.items[0], set(['a']))
        self.assertEqual(buckets.items[1], set(['a','c']))
        self.assertEqual(buckets.items[2], set(['c']))
        self.assertEqual(buckets.items[3], set(['b','c']))
        self.assertEqual(buckets.items[4], set(['b']))
        buckets.remove(Range.closed(9,20))
        self.assertEqual(len(buckets),5)
        self.assertEqual(buckets.ranges[0], Range.open(3,4))
        self.assertEqual(buckets.ranges[1], Range.closed(4,5))
        self.assertEqual(buckets.ranges[2], Range.open(5,7))
        self.assertEqual(buckets.ranges[3], Range.closed(7,8))
        self.assertEqual(buckets.ranges[4], Range.open(8,9))
        self.assertEqual(buckets.items[0], set(['a']))
        self.assertEqual(buckets.items[1], set(['a','c']))
        self.assertEqual(buckets.items[2], set(['c']))
        self.assertEqual(buckets.items[3], set(['b','c']))
        self.assertEqual(buckets.items[4], set(['b']))
        buckets.remove(Range.closed(5,7))
        self.assertEqual(len(buckets),4)
        self.assertEqual(buckets.ranges[0], Range.open(3,4))
        self.assertEqual(buckets.ranges[1], Range.closedOpen(4,5))
        self.assertEqual(buckets.ranges[2], Range.openClosed(7,8))
        self.assertEqual(buckets.ranges[3], Range.open(8,9))
        self.assertEqual(buckets.items[0], set(['a']))
        self.assertEqual(buckets.items[1], set(['a','c']))
        self.assertEqual(buckets.items[2], set(['b','c']))
        self.assertEqual(buckets.items[3], set(['b']))
    def test_iteritems(self):
        if debug: print("Testing iteritems")
        buckets = RangeBucketMap()
        buckets.put(Range.closed(3,5),'a')
        buckets.put(Range.closed(7,10),'b')
        buckets.put(Range.closed(4,8),'c')
        iterator = buckets.iteritems(2,10)
        self.assertEquals(next(iterator), (Range.closed(3,5), 'a'))
        self.assertEquals(next(iterator), (Range.closed(4,8), 'c'))
        self.assertEquals(next(iterator), (Range.closed(7,10), 'b'))
        with self.assertRaises(StopIteration):
            next(iterator)
        iterator = buckets.iteritems(3,8)
        self.assertEquals(next(iterator), (Range.closed(3,5), 'a'))
        self.assertEquals(next(iterator), (Range.closed(4,8), 'c'))
        self.assertEquals(next(iterator), (Range.closed(7,8), 'b'))
        with self.assertRaises(StopIteration):
            next(iterator)        
if __name__ == "__main__":
    debug = True
    unittest.main(exit = False)



import unittest
from Ranger.src.Collections.RangeMap import RangeMap
from Ranger.src.Range.Range import Range

debug = False

class RangeMapTest(unittest.TestCase):
    """ Unit Tests for RangeMap.py """
    def test_contains(self):
        if debug: print("Testing contains")
        theMap = RangeMap()
        theMap.put(Range.closed(3,5),'foo')
        theMap.put(Range.closed(7,10),'bar')
        self.assertTrue(theMap.contains(4))
        self.assertFalse(theMap.contains(2))
        self.assertTrue(theMap.contains(Range.closed(4,5)))
        self.assertTrue(theMap.contains(Range.closed(8,9)))
        self.assertFalse(theMap.contains(Range.closed(1,4)))
        self.assertFalse(theMap.contains(Range.closed(6,6)))
        self.assertFalse(theMap.contains(Range.closed(8,12)))
    def test_overlaps(self):
        if debug: print("Testing overlaps")
        theMap = RangeMap()
        theMap.put(Range.closed(3,5),'foo')
        theMap.put(Range.closed(7,10),'bar')
        self.assertTrue(theMap.overlaps(4))
        self.assertFalse(theMap.overlaps(Range.closedOpen(1,3)))
        self.assertFalse(theMap.overlaps(Range.open(5,7)))
        self.assertFalse(theMap.overlaps(2))
        self.assertTrue(theMap.overlaps(3))
        self.assertTrue(theMap.overlaps(10))
        self.assertTrue(theMap.overlaps(Range.closed(4,5)))
        self.assertTrue(theMap.overlaps(Range.closed(8,9)))
        self.assertTrue(theMap.overlaps(Range.closed(1,4)))
        self.assertFalse(theMap.overlaps(Range.closed(6,6)))
        self.assertTrue(theMap.overlaps(Range.closed(8,12)))
        self.assertTrue(theMap.overlaps(Range.closed(1,12)))    
    def test_put(self):
        if debug: print("Testing put")
        rangeMap = RangeMap()
        rangeMap.put(Range.closed(1,10),'foo')
        self.assertEqual(rangeMap.ranges[0], Range.closed(1,10))
        self.assertEqual(rangeMap.items[0], 'foo')
        rangeMap.put(Range.open(3,6), 'bar')
        self.assertEqual(rangeMap.ranges[0], Range.closed(1,3))
        self.assertEqual(rangeMap.ranges[1], Range.open(3,6))
        self.assertEqual(rangeMap.ranges[2], Range.closed(6,10))
        self.assertEqual(rangeMap.items[0],'foo')
        self.assertEqual(rangeMap.items[1], 'bar')
        self.assertEqual(rangeMap.items[2],'foo')
        rangeMap.put(Range.open(10,20), 'foo')
        self.assertEqual(len(rangeMap),4)        
        self.assertEqual(rangeMap.ranges[3], Range.open(10,20))
        self.assertEqual(rangeMap.items[0],'foo')
        self.assertEqual(rangeMap.items[1], 'bar')
        self.assertEqual(rangeMap.items[2],'foo')
        self.assertEqual(rangeMap.items[3],'foo')
    def test_remove(self):
        if debug: print("Testing remove")
        rangeMap = RangeMap()
        rangeMap.put(Range.closed(1,10),'foo')
        rangeMap.put(Range.open(3,6), 'bar')
        rangeMap.put(Range.open(10,20), 'foo')
        rangeMap.remove(Range.closed(5,11))
        self.assertEqual(len(rangeMap), 3)
        self.assertEqual(rangeMap.ranges[0], Range.closed(1,3))
        self.assertEqual(rangeMap.ranges[1], Range.open(3,5))
        self.assertEqual(rangeMap.ranges[2], Range.open(11,20))
        self.assertEqual(rangeMap.items[0],'foo')
        self.assertEqual(rangeMap.items[1], 'bar')
        self.assertEqual(rangeMap.items[2],'foo')
        rangeMap.remove(Range.closed(0,1))
        self.assertEqual(len(rangeMap), 3)
        self.assertEqual(rangeMap.ranges[0], Range.openClosed(1,3))
        self.assertEqual(rangeMap.ranges[1], Range.open(3,5))
        self.assertEqual(rangeMap.ranges[2], Range.open(11,20))
        self.assertEqual(rangeMap.items[0],'foo')
        self.assertEqual(rangeMap.items[1], 'bar')
        self.assertEqual(rangeMap.items[2],'foo')
    def test_get(self):
        if debug: print("Testing get")
        rangeMap = RangeMap()
        rangeMap.put(Range.closed(1,10),'foo')
        rangeMap.put(Range.open(3,6), 'bar')
        rangeMap.put(Range.open(10,20), 'foo')
        self.assertEqual(rangeMap.get(1),set(['foo']))
        self.assertEquals(rangeMap.get(4),set(['bar']))
        with self.assertRaises(KeyError):
            rangeMap.get(20)
        self.assertEqual(rangeMap.get(Range.closed(11,15)),set(['foo']))
        self.assertEquals(rangeMap.get(Range.closed(5,15)),set(['foo','bar']))
        with self.assertRaises(KeyError):
            rangeMap.get(Range.closed(20,30))
        self.assertEquals(rangeMap.get(Range.closed(15,100)),set(['foo']))
if __name__ == "__main__":
    debug = True
    unittest.main(exit = False)



import unittest
from Ranger.test.src.Range.RangeTestSuite import RangeTestSuite
from Ranger.test.src.Collections.CollectionsTestSuite import CollectionsTestSuite

class mainTestSuite(unittest.TestSuite):
    def __init__(self):
        super(mainTestSuite, self).__init__()
        self.addTest(RangeTestSuite())
        self.addTest(CollectionsTestSuite())

if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(mainTestSuite())



import unittest
from Ranger.test.src.Range.CutTest import CutTest
from Ranger.test.src.Range.RangeTest import RangeTest

class RangeTestSuite(unittest.TestSuite):
    def __init__(self):
        super(RangeTestSuite, self).__init__()
        self.addTest(unittest.makeSuite(CutTest))
        self.addTest(unittest.makeSuite(RangeTest))

if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(RangeTestSuite())



from bisect import bisect_left
from collections import deque, Hashable
from Ranger.src.Collections.RangeMap import RangeMap
from Ranger.src.Range.Range import Range
from Ranger.src.Range.Cut import Cut

class RangeBucketMap(RangeMap):
    """ Class used to represent a mapping of disjoint ranges to sets of items. Ranges
    do not coalesce. However, if a new Range is added over an existing Range, items
    belonging to the existing Range are retained in that Range
    """
    def __init__(self, rangeDict = None):
        """ Instantiates a RangeBucketMap

        Parameters
        ----------
        rangeDict : Dictionary of Range -> object
            Dictionary to start off the RangeBucketMap with
        """
        self.recurseAdd = False
        super(RangeBucketMap, self).__init__(rangeDict)
    def iteritems(self, start = None, end = None):
        """ Iterates over pairs of (Range, value)

        Parameters
        ----------
        start : comparable, optional
            The starting point for iterating, inclusive
        end : comparable, optional
            The ending point for iterating, inclusive

        Returns
        -------
        Generator of (Range intersecting [start,end], value), ordered by start point
        """
        if start is None:
            start = self.lower_cuts[0]
        else:
            start = Cut.belowValue(start)
        if end is None:
            end = self.upper_cuts[-1]
        else:
            end = Cut.aboveValue(end)
        bounding_range = Range(start, end)
        # Get the bounding indices
        ovlapLowerInd = max(bisect_left(self.lower_cuts, start)-1,0)
        ovlapUpperInd = bisect_left(self.lower_cuts, end)
        # Create queue of values that need to be generated
        yield_vals = deque()
        # Create dictionary of values to be generated -> indices containing them
        vals_inds_dict = {}
        for i in range(ovlapLowerInd, ovlapUpperInd):
            # Check if anything can be released from the queue
            while len(yield_vals) > 0:
                if vals_inds_dict[yield_vals[0]][-1] < i-1:
                    # Yield the full range, value. Remove value from queue
                    val = yield_vals.popleft()
                    yield Range(max(self.lower_cuts[vals_inds_dict[val][0]],start),
                                min(self.upper_cuts[vals_inds_dict[val][-1]],end)), val
                    # Remove value from dict
                    del vals_inds_dict[val]
                else:
                    break
            try:
                # Get intersection of the ranges
                intersect = bounding_range.intersection(self.ranges[i])
                if not intersect.isEmpty():
                    # If overlapping with this range, put into queue
                    for val in self.items[i]:
                        if val not in vals_inds_dict:
                            yield_vals.append(val)
                            vals_inds_dict[val] = deque()
                        vals_inds_dict[val].append(i)
            except ValueError:
                # Continue if no overlap with this range
                continue
        ## Yield remaining values
        while len(yield_vals) > 0:
            # Yield the full range, value. Remove value from queue
            val = yield_vals.popleft()
            yield Range(max(self.lower_cuts[vals_inds_dict[val][0]],start),
                        min(self.upper_cuts[vals_inds_dict[val][-1]],end)), val
            # Remove value from dict
            del vals_inds_dict[val]
                
    def get(self, key):
        """ Get the item(s) corresponding to a given key. The key can be a
        Range or a single value that is within a Range

        Parameters
        ----------
        key : comparable
            A single value or Range object

        Raises
        ------
        KeyError
            If there is no overlap with the key
        ValueError
            If the key type not compatible with the ranges
        
        Returns
        -------
        A set containing all overlapping items
        """
        if not self.overlaps(key):
            self.__missing__(key)
        elif isinstance(key, Range):
            # If this is a single value
            returnSet = set()
            # Get the bounding indices
            ovlapLowerInd = max(bisect_left(self.lower_cuts, key.lowerCut)-1,0)
            ovlapUpperInd = bisect_left(self.lower_cuts, key.upperCut)
            for i in range(ovlapLowerInd, ovlapUpperInd):
                try:
                    # Get intersection of the ranges
                    intersect = key.intersection(self.ranges[i])
                    if not intersect.isEmpty():
                        # If overlapping with this range, put its
                        # item in the return set
                        returnSet = returnSet.union(self.items[i])
                except ValueError:
                    # Continue if no overlap with this range
                    continue
            # Return the set of items
            return returnSet
        else:
            # If this is a single value
            # Get the index of the range containing the value
            lower_ind = max(bisect_left(self.lower_cuts, key)-1,0)
            # Return the item at that value
            return self.items[lower_ind]  
    def put(self, key, val):
        """ Creates a mapping from a Range to a value, adding to
        any existing values over that Range

        Parameters
        ----------
        key : Range object
            A Range to serve as a key
        val : value, hashable
            Some value that the Range should map to

        Raises
        ------
        TypeError
            If the key is not a Range object or value is not hashable
        """
        if not isinstance(key, Range):
            raise TypeError("key is not a Range")
        elif not any((isinstance(val, Hashable), self.recurseAdd)):
            raise TypeError("value not hashable")
        elif key.isEmpty():
            # Skip if this is an empty range
            return
        # Figure out where to the key/value
        if not self.overlaps(key):
            # If this range is completely on its own, just insert
            insertInd = bisect_left(self.lower_cuts, key.lowerCut)
            self.ranges.insert(insertInd, key)
            self.lower_cuts.insert(insertInd, key.lowerCut)
            self.upper_cuts.insert(insertInd, key.upperCut)
            if not isinstance(val, set):
                self.items.insert(insertInd, set([val]))
            else:
                self.items.insert(insertInd, val)
            return
        else:
            # If this range has some overlap with existing ranges
            ovlapLowerInd = max(bisect_left(self.lower_cuts, key.lowerCut)-1,0)
            ovlapUpperInd = bisect_left(self.lower_cuts, key.upperCut)
            # Create queue ranges to add
            addRanges = deque()
            # Create queue of items to add
            addItems = deque()
            # Keep track of next lower cutpoint to add
            nextLowerCut = key.lowerCut
            for i in range(ovlapLowerInd, ovlapUpperInd):
                try:
                    # Get intersection of the ranges
                    intersect = key.intersection(self.ranges[i])
                    if not intersect.isEmpty():
                        # Add in a Range between the next LowerCut and
                        # the beginning of this intersection if necessary
                        if nextLowerCut < intersect.lowerCut:
                            addRanges.append(Range(nextLowerCut, intersect.lowerCut))
                            addItems.append(val)
                            nextLowerCut = intersect.lowerCut
                        if intersect == self.ranges[i]:
                            ## If key encompassing existing Range ##
                            # Add item to this range
                            self.items[i].add(val)
                            # Change the next lower cut
                            nextLowerCut = intersect.upperCut
                        elif self.lower_cuts[i] == intersect.lowerCut:
                            ## If key upper cutpoint enclosed by existing Range ##
                            # Add in the rest of the original Range
                            if self.upper_cuts[i] > intersect.upperCut:
                                addRanges.append(Range(intersect.upperCut,
                                                       self.upper_cuts[i]))
                                addItems.append(set(self.items[i]))
                            # Define original part to be shorter                            
                            self.upper_cuts[i] = intersect.upperCut
                            self.ranges[i] = Range(self.lower_cuts[i],
                                                   intersect.upperCut)
                            self.items[i].add(val)
                            # Change the next lower cut
                            nextLowerCut = intersect.upperCut
                        elif self.upper_cuts[i] == intersect.upperCut:
                            ## If key lower cutpoint enclosed by existing Range ##
                            # Add in the rest of the original Range
                            if intersect.lowerCut > self.lower_cuts[i]:
                                addRanges.append(Range(self.lower_cuts[i], intersect.lowerCut))
                                addItems.append(set(self.items[i]))
                            # Define original part to be shorter
                            self.lower_cuts[i] = intersect.lowerCut
                            self.ranges[i] = Range(self.lower_cuts[i],
                                                   intersect.upperCut)
                            self.items[i].add(val)
                            # Change the next lower cut
                            nextLowerCut = intersect.upperCut
                        else:
                            # If entire key enclosed by existing Range
                            # Add in lower part of original Range
                            addRanges.append(Range(self.lower_cuts[i], intersect.lowerCut))
                            addItems.append(set(self.items[i]))
                            # Add in upper part of original Range
                            addRanges.append(Range(intersect.upperCut, self.upper_cuts[i]))
                            addItems.append(set(self.items[i]))
                            # Define original part to be middle
                            self.lower_cuts[i] = intersect.lowerCut
                            self.upper_cuts[i] = intersect.upperCut
                            self.ranges[i] = Range(intersect.lowerCut,intersect.upperCut)
                            self.items[i].add(val)
                            # Change the next lower cut
                            nextLowerCut = intersect.upperCut
                except ValueError:
                    # Continue if no overlap with this range
                    continue
            # Put in a last range if necessary
            if nextLowerCut < key.upperCut:
                addRanges.append(Range(nextLowerCut, key.upperCut))
                addItems.append(val)
            # Use recursive call to place the pairs, which now
            # should not overlap with any other ranges
            self.recurseAdd = True
            while len(addRanges) > 0:
                self.put(addRanges.pop(),addItems.pop())
            self.recurseAdd = False
    def remove(self, aRange):
        """ Removes a range and its value(s) from the range set

        Parameters
        ----------
        aRange : A Range object
            The Range to remove

        Raises
        ------
        ValueError
            If removing range of type not compatible with previously
            added ranges
        TypeError
            If not a Range
        """
        if not isinstance(aRange, Range):
            raise TypeError("aRange is not a Range")
        elif aRange.isEmpty():
            # Skip if this is an empty range
            return
        # Check for compatibility of types if necessary
        if len(self) > 0:
            if not (issubclass(aRange.lowerCut.theType,
                               self.ranges[0].lowerCut.theType) or \
                    issubclass(self.ranges[0].lowerCut.theType,
                               aRange.lowerCut.theType)):
                raise ValueError("Range not compatible with previously added ranges")
        # Check if the range actually overlaps with the key set
        if not self.overlaps(aRange):
            return
        else:
            # There's some overlap, so deal with that
            # Determine where overlap occurs
            ovlapLowerInd = max(bisect_left(self.lower_cuts,
                                            aRange.lowerCut)-1,0)
            ovlapUpperInd = bisect_left(self.lower_cuts, aRange.upperCut)
            # Create queue of indices marked for removal
            removeRanges = deque()
            # Create queue of ranges to add
            addRanges = deque()
            # Create queue of items to add with the addRanges
            addItems = deque()
            for i in range(ovlapLowerInd, ovlapUpperInd):
                try:
                    # Get intersection of the ranges
                    intersect = aRange.intersection(self.ranges[i])
                    if not intersect.isEmpty():
                        if intersect == self.ranges[i]:
                            # Mark range for removal
                            removeRanges.append(i)
                        elif self.lower_cuts[i] == intersect.lowerCut:
                            # If equal on the left cutpoint, subtract
                            # out left part
                            self.lower_cuts[i] = intersect.upperCut
                            self.ranges[i] = Range(intersect.upperCut,
                                                   self.upper_cuts[i])
                        elif self.upper_cuts[i] == intersect.upperCut:
                            # If equal on right cutpoint, subtract out
                            # right part
                            self.upper_cuts[i] = intersect.lowerCut
                            self.ranges[i] = Range(self.lower_cuts[i],
                                                   intersect.lowerCut)
                        else:
                            # If in the middle, split into two parts, putting
                            # both in add queue and placing the old range index
                            # into the remove queue
                            addRanges.append(Range(self.lower_cuts[i],
                                                   intersect.lowerCut))
                            addItems.append(set(self.items[i]))
                            addRanges.append(Range(intersect.upperCut,
                                                   self.upper_cuts[i]))
                            addItems.append(set(self.items[i]))
                            removeRanges.append(i)
                except ValueError:
                    # Continue if no overlap with this range
                    continue
            # Remove any ranges that are marked for removal
            while len(removeRanges) > 0:
                removeInd = removeRanges.pop()
                self.ranges.pop(removeInd)
                self.lower_cuts.pop(removeInd)
                self.upper_cuts.pop(removeInd)
                self.items.pop(removeInd)
            # Add any pairs that need to be added
            self.recurseAdd = True
            while len(addRanges) > 0:
                self.put(addRanges.pop(), addItems.pop())        
            self.recurseAdd = False



""" Release data for Ranger (based on release.py from Networkx library).
"""

from __future__ import absolute_import
import os
import sys
import time
import datetime

basedir = os.path.abspath(os.path.split(__file__)[0])

def write_versionfile():
    """Creates a static file containing version information."""
    versionfile = os.path.join(basedir, 'version.py')

    text = '''"""
Version information for Ranger, created during installation.

Do not add this file to the repository.

"""

import datetime

version = %(version)r
date = %(date)r

# Was Ranger built from a development version? If so, remember that the major
# and minor versions reference the "target" (rather than "current") release.
dev = %(dev)r

# Format: (name, major, min, revision)
version_info = %(version_info)r

# Format: a 'datetime.datetime' instance
date_info = %(date_info)r

# Format: (vcs, vcs_tuple)
vcs_info = %(vcs_info)r

'''

    # Try to update all information
    date, date_info, version, version_info, vcs_info = get_info(dynamic=True)

    def writefile():
        fh = open(versionfile, 'w')
        subs = {
            'dev' : dev,
            'version': version,
            'version_info': version_info,
            'date': date,
            'date_info': date_info,
            'vcs_info': vcs_info
        }
        fh.write(text % subs)
        fh.close()

    if vcs_info[0] == 'mercurial':
        # Then, we want to update version.py.
        writefile()
    else:
        if os.path.isfile(versionfile):
            # This is *good*, and the most likely place users will be when
            # running setup.py. We do not want to overwrite version.py.
            # Grab the version so that setup can use it.
            sys.path.insert(0, basedir)
            from version import version
            del sys.path[0]
        else:
            # This is *bad*.  It means the user might have a tarball that
            # does not include version.py.  Let this error raise so we can
            # fix the tarball.
            ##raise Exception('version.py not found!')

            # We no longer require that prepared tarballs include a version.py
            # So we use the possibly trunctated value from get_info()
            # Then we write a new file.
            writefile()

    return version

def get_revision():
    """Returns revision and vcs information, dynamically obtained."""
    vcs, revision, tag = None, None, None

    hgdir = os.path.join(basedir, '..', '.hg')
    gitdir = os.path.join(basedir, '..', '.git')

    if os.path.isdir(gitdir):
        vcs = 'git'
        # For now, we are not bothering with revision and tag.

    vcs_info = (vcs, (revision, tag))

    return revision, vcs_info

def get_info(dynamic=True):
    ## Date information
    date_info = datetime.datetime.now()
    date = time.asctime(date_info.timetuple())

    revision, version, version_info, vcs_info = None, None, None, None

    import_failed = False
    dynamic_failed = False

    if dynamic:
        revision, vcs_info = get_revision()
        if revision is None:
            dynamic_failed = True

    if dynamic_failed or not dynamic:
        # This is where most final releases of NetworkX will be.
        # All info should come from version.py. If it does not exist, then
        # no vcs information will be provided.
        sys.path.insert(0, basedir)
        try:
            from version import date, date_info, version, version_info, vcs_info
        except ImportError:
            import_failed = True
            vcs_info = (None, (None, None))
        else:
            revision = vcs_info[1][0]
        del sys.path[0]

    if import_failed or (dynamic and not dynamic_failed):
        # We are here if:
        #   we failed to determine static versioning info, or
        #   we successfully obtained dynamic revision info
        version = ''.join([str(major), '.', str(minor)])
        if dev:
            version += '.dev_' + date_info.strftime("%Y%m%d%H%M%S")
        version_info = (name, major, minor, revision)

    return date, date_info, version, version_info, vcs_info

## Version information
name = 'Ranger'
major = "0"
minor = "1"


## Declare current release as a development release.
## Change to False before tagging a release; then change back.
dev = True


description = "A Python package for the manipulation of range objects"

long_description = \
"""
Ranger is a Python package for the manipulation of range objects.
Ranges may extend over discrete or continuous domains, be open or closed,
bounded or unbounded. Ranger also includes tools for managing collections
 of ranges, which may be mapped to other objects.
"""
license = 'BSD'
authors = {'Rodgers-Melnick' : ('Eli Rodgers-Melnick','er432@cornell.edu'),
           }
url = 'https://github.com/er432/Ranger'
platforms = ['Linux','Mac OSX','Windows','Unix']
keywords = ['Ranges','Set','Set theory']
classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Information Analysis'
    ]

date, date_info, version, version_info, vcs_info = get_info()

if __name__ == '__main__':
    # Write versionfile for nightly snapshots.
    write_versionfile()




#!/usr/bin/env python
from setuptools import setup, find_packages
import os
import sys

packages = ['Ranger',
            'Ranger.src',
            'Ranger.src.Range',
            'Ranger.src.Collections',
            'Ranger.test',
            'Ranger.test.src',
            'Ranger.test.src.Range',
            'Ranger.test.src.Collections'
           ]
setup(
    name = 'ranger',
    version = '0.8',
    packages=packages,
    author = 'Eli Rodgers-Melnick',
    author_email = 'er432@cornell.edu',
    description = 'A Python package for the manipulation of Range objects',
    keywords = ['Ranges','Set','Set theory'],
    long_description = """
Ranger is a Python package for the manipulation of range objects.
Ranges may extend over discrete or continuous domains, be open or closed,
bounded or unbounded. Ranger also includes tools for managing collections
 of ranges, which may be mapped to other objects.
""",
    license = 'BSD',
    platforms = ['Linux','Mac OSX','Windows','Unix'],
    url = 'https://github.com/er432/Ranger',
    test_suite = 'Ranger.test',
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Information Analysis'
    ]
    )



__author__ = 'Eli Rodgers-Melnick'
__license__ = 'BSD'

# Make imports
from Ranger.src.Range.Range import Range
from Ranger.src.Collections.RangeSet import RangeSet
from Ranger.src.Collections.RangeMap import RangeMap
from Ranger.src.Collections.RangeBucketMap import RangeBucketMap



from bisect import bisect_left
from collections import deque
from Ranger.src.Range.Range import Range

class RangeSet(object):
    """ Class used to represent a set of non-overlapping ranges of the
    same type. If a range is added that is connected to another range
    already in the set, those ranges are merged. Otherwise, it is added as
    a new range in the set
    """
    def __init__(self, ranges = None):
        """ Instantiates the RangeSet

        Parameters
        ----------
        ranges : List of Range objects
            Ranges to add to the Set
        """
        ## Holds lower and upper cut points of ranges
        self.lower_cuts = []
        self.upper_cuts = []
        ## Holds the range objects in the set
        self.ranges = []
        if ranges is not None:
            for aRange in ranges:
                self.add(aRange)
    def __repr__(self):
        if len(self) < 6:
            return "RangeSet(%s)" % ", ".join(map(str, self.ranges))
        else:
            return "RangeSet(%s, ..., %s)" % (", ".join(map(str, self.ranges[:2])),
                                              ", ".join(map(str, self.ranges[-2:])))
    def __len__(self):
        return len(self.ranges)
    def __iter__(self):
        return iter(self.ranges)
    def __eq__(self, other):
        if not isinstance(other, RangeSet):
            return False
        elif len(self) != len(other):
            return False
        else:
            for r1,r2 in zip(self.ranges, other.ranges):
                if r1 != r2: return False
            return True
    def __ne__(self, other):
        return not self.__eq__(other)
    def add(self, aRange):
        """ Adds a range to the range set. If this range is not connected
        to any current ranges, it will place the new range on its own. If
        there is a connection, any connected ranges will be merged into a single
        range

        Parameters
        ----------
        aRange : A Range object
            The Range to add to the RangeSet

        Raises
        ------
        ValueError
            If adding range of type not compatible with previously
            added ranges
        TypeError:
            If not adding a Range
        """
        if not isinstance(aRange, Range):
            raise TypeError("aRange is not a Range")
        elif aRange.isEmpty():
            # Skip if this is an empty range
            return
        # Check for compatibility of types if necessary
        if len(self) > 0:
            if not (issubclass(aRange.lowerCut.theType,
                               self.ranges[0].lowerCut.theType) or \
                    issubclass(self.ranges[0].lowerCut.theType,
                               aRange.lowerCut.theType)):
                raise ValueError("Range not compatible with previously added ranges")
        # Get the insertion point (where the lower bound should go), should
        # this range be added on its own
        lower_ind = bisect_left(self.lower_cuts, aRange.lowerCut)
        if len(self) == 0:
            # Add on its own if there is nothing in the list
            self.ranges.append(aRange)
            self.lower_cuts.append(aRange.lowerCut)
            self.upper_cuts.append(aRange.upperCut)
        elif len(self) == lower_ind:
            if not aRange.isConnected(self.ranges[max(lower_ind-1,0)]):
                # Add on its own if not connected to previous and last
                self.ranges.insert(lower_ind,aRange)
                self.lower_cuts.insert(lower_ind,aRange.lowerCut)
                self.upper_cuts.insert(lower_ind,aRange.upperCut)
            else:
                # If connected with the range below, replace with new range
                newLowerCut = min(aRange.lowerCut,
                                  self.lower_cuts[max(lower_ind-1,0)])
                newUpperCut = max(aRange.upperCut,
                                  self.upper_cuts[max(lower_ind-1,0)])
                newRange = Range(newLowerCut, newUpperCut)
                self.ranges[-1] = newRange
                self.lower_cuts[-1] = newLowerCut
                self.upper_cuts[-1] = newUpperCut
        elif not any((aRange.isConnected(self.ranges[max(lower_ind-1,0)]),
                      aRange.isConnected(self.ranges[lower_ind]))):
            # Add on its own if not connected
            self.ranges.insert(lower_ind,aRange)
            self.lower_cuts.insert(lower_ind,aRange.lowerCut)
            self.upper_cuts.insert(lower_ind,aRange.upperCut)
        elif aRange.isConnected(self.ranges[max(lower_ind-1,0)]):
            # If connected with range below
            newLowerCut = min(self.lower_cuts[max(lower_ind-1,0)],
                              aRange.lowerCut)
            newUpperCut = max(aRange.upperCut,
                              self.upper_cuts[max(lower_ind-1,0)])
            removeCount = 1
            if len(self) == (lower_ind):
                # If hitting the last range, take the maximum uppercut
                newUpperCut = max(newUpperCut, self.upper_cuts[max(lower_ind-1,0)])
            else:
                # If not hitting the last range, go find the upper cut
                for i in range(max(1,lower_ind), len(self)):
                    if aRange.isConnected(self.ranges[i]):
                        newUpperCut = max(newUpperCut,self.upper_cuts[i])
                        removeCount += 1
                    else:
                        break
            # Make the new range
            newRange = Range(newLowerCut, newUpperCut)
            # Get rid of all overlapping ranges
            for i in range(removeCount):
                self.ranges.pop(max(lower_ind-1,0))
                self.lower_cuts.pop(max(lower_ind-1,0))
                self.upper_cuts.pop(max(lower_ind-1,0))
            # Add the new range
            self.ranges.insert(max(lower_ind-1,0),newRange)
            self.lower_cuts.insert(max(lower_ind-1,0),newRange.lowerCut)
            self.upper_cuts.insert(max(lower_ind-1,0),newRange.upperCut)
        elif aRange.isConnected(self.ranges[lower_ind]):
            # If connected with the range above
            newLowerCut = min(aRange.lowerCut, self.lower_cuts[lower_ind])
            newUpperCut = max(aRange.upperCut, self.upper_cuts[lower_ind])
            removeCount = 0
            if len(self) == (lower_ind + 1):
                # If hitting the last range, you're done
                removeCount += 1
            else:
                # Go find the upper cut
                for i in range(lower_ind, len(self)):
                    if aRange.isConnected(self.ranges[i]):
                        newUpperCut = max(newUpperCut, self.upper_cuts[i])
                        removeCount += 1
                    else:
                        break
            # Make the new range
            newRange = Range(newLowerCut, newUpperCut)
            # Remove the overlapping ranges
            for i in range(removeCount):
                self.ranges.pop(lower_ind)
                self.lower_cuts.pop(lower_ind)
                self.upper_cuts.pop(lower_ind)
            # Add the new range
            self.ranges.insert(lower_ind, newRange)
            self.lower_cuts.insert(lower_ind, newRange.lowerCut)
            self.upper_cuts.insert(lower_ind, newRange.upperCut)

    def contains(self, val):
        """ Returns true if any of the ranges fully enclose the given
        value, which can be a single value or a Range object

        Parameters
        ----------
        val : A single value or a Range object

        Raises
        ------
        ValueError
            If the value type not compatible with the ranges
        
        Returns
        -------
        true if any of the ranges fully enclose the given value
        """
        if len(self) == 0: return False
        # Get the index+1 of the highest lower cut <= to the value or its
        # lower cutpoint and check if the value contained
        if isinstance(val, Range):
            lower_ind = max(bisect_left(self.lower_cuts, val.lowerCut)-1,0)
            return self.ranges[lower_ind].encloses(val)
        else:
            lower_ind = max(bisect_left(self.lower_cuts, val)-1,0)
            return self.ranges[lower_ind].contains(val)
    def difference(self, otherSet):
        """ Creates a new RangeSet in which all elements in another RangeSet
        are taken out of this RangeSet

        Parameters
        ----------
        otherSet : RangeSet object
            The RangeSet used for this difference

        Raises
        ------
        TypeError
            If the object passed in is not a RangeSet
        ValueError
            If the value type of the ranges in the other set not compatible
            with the range's values

        Returns
        -------
        RangeSet consisting of the difference of the two sets
        """
        if not isinstance(otherSet, RangeSet):
            raise TypeError("otherSet is not a RangeSet")
        newSet = RangeSet()
        for addRange in self.ranges:
            if otherSet.overlaps(addRange):
                # Determine where overlap occurs
                otherLowerInd = max(bisect_left(otherSet.lower_cuts,
                                            addRange.lowerCut)-1,0)
                otherUpperInd = bisect_left(otherSet.lower_cuts,
                                            addRange.upperCut)
                newLowerCut = addRange.lowerCut
                newUpperCut = addRange.upperCut
                add = True
                for i in range(otherLowerInd, otherUpperInd):
                    try:
                        # Get the intersection of the ranges
                        intersect = addRange.intersection(otherSet.ranges[i])
                        if not intersect.isEmpty():
                            if addRange == intersect:
                                add = False
                                break
                            elif addRange.lowerCut == intersect.lowerCut:
                                # If equal on the left cutpoint, subtract out left
                                # part
                                newLowerCut = intersect.upperCut
                                addRange = Range(newLowerCut, newUpperCut)
                            elif addRange.upperCut == intersect.upperCut:
                                # If equal on right cutpoint, subtract out right
                                # part
                                newUpperCut = intersect.lowerCut
                                addRange = Range(addRange.lowerCut, newUpperCut)
                            else:
                                # If in the middle, split into two parts and
                                # add the lower one immediately
                                newSet.add(Range(newLowerCut,
                                                 intersect.lowerCut))
                                newLowerCut = intersect.upperCut
                                newUpperCut = addRange.upperCut
                                addRange = Range(newLowerCut, newUpperCut)
                    except ValueError:
                        continue
                if add:
                    newSet.add(Range(newLowerCut, newUpperCut))
            else:
                newSet.add(addRange)
        return newSet
    def intersection(self, otherSet):
        """ Creates a new RangeSet that is the intersection of this and
        another RangeSet

        Parameters
        ----------
        otherSet : RangeSet object
            The RangeSet used for this intersection

        Raises
        ------
        TypeError
            If the object passed in is not a RangeSet
        ValueError
            If the value type of the ranges in the other set not compatible
            with the range's values

        Returns
        -------
        RangeSet consisting of the intersection of the two sets
        """
        if not isinstance(otherSet, RangeSet):
            raise TypeError("otherSet is not a RangeSet")
        newSet = RangeSet()
        for addRange in self.ranges:
            if otherSet.overlaps(addRange):
                # Determine where overlap occurs
                otherLowerInd = max(bisect_left(otherSet.lower_cuts,
                                                addRange.lowerCut)-1,0)
                otherUpperInd = bisect_left(otherSet.lower_cuts,
                                            addRange.upperCut)
                for i in range(otherLowerInd, otherUpperInd):
                    # Get the intersection of the ranges
                    try:
                        intersect = addRange.intersection(otherSet.ranges[i])
                        # Add intersection if there is any overlap
                        if not intersect.isEmpty():
                            newSet.add(intersect)
                    except ValueError:
                        continue
        return newSet
    def overlaps(self, val):
        """ Returns true if any of the ranges at least partially overlap
        the given value, which can be a single value or a Range object

        Parameters
        ----------
        val : A single value or a Range object

        Raises:
        -------
        ValueError
            If the value type not compatible with the ranges

        Returns
        -------
        true if any of the ranges fully enclose the given value
        """
        if len(self) == 0: return False
        # Get the index+1 of the highest lower cut <= to the value or its
        # lower cutpoint and check if the value overlaps
        if isinstance(val, Range):
            lower_ind = bisect_left(self.lower_cuts, val.lowerCut)-1
            upper_ind = bisect_left(self.lower_cuts, val.upperCut)
            for i in range(lower_ind,upper_ind):
                if val.isConnected(self.ranges[i]):
                    if not self.ranges[i].intersection(val).isEmpty():
                        return True
            return False
        else:
            lower_ind = bisect_left(self.lower_cuts,val)-1
            return self.ranges[lower_ind].contains(val)
    def remove(self, aRange):
        """ Removes a range from the range set. 

        Parameters
        ----------
        aRange : A Range object
            The Range to remove from the RangeSet

        Raises
        ------
        ValueError
            If removing range of type not compatible with previously
            added ranges
        TypeError
            If not a Range 
        """
        if not isinstance(aRange, Range):
            raise TypeError("aRange is not a Range")
        elif aRange.isEmpty():
            # Skip if this is an empty range
            return
        # Check for compatibility of types if necessary
        if len(self) > 0:
            if not (issubclass(aRange.lowerCut.theType,
                               self.ranges[0].lowerCut.theType) or \
                    issubclass(self.ranges[0].lowerCut.theType,
                               aRange.lowerCut.theType)):
                raise ValueError("Range not compatible with previously added ranges")
        # Check if the range actually overlaps with this set
        if not self.overlaps(aRange):
            return
        else:
            # There's some overlap, so deal with that
            # Determine where overlap occurs
            ovlapLowerInd = max(bisect_left(self.lower_cuts, aRange.lowerCut)-1,0)
            ovlapUpperInd = bisect_left(self.lower_cuts, aRange.upperCut)
            # Create queue of indices marked for removal
            removeRanges = deque()
            # Create queue of ranges to add
            addRanges = deque()
            for i in range(ovlapLowerInd, ovlapUpperInd):
                try:
                    # Get intersection of the ranges
                    intersect = aRange.intersection(self.ranges[i])
                    if not intersect.isEmpty():
                        if intersect == self.ranges[i]:
                            # Mark range for removal
                            removeRanges.append(i)
                        elif self.lower_cuts[i] == intersect.lowerCut:
                            # If equal on the left cutpoint, subtract out left
                            # part
                            self.lower_cuts[i] = intersect.upperCut
                            self.ranges[i] = Range(intersect.upperCut, self.upper_cuts[i])
                        elif self.upper_cuts[i] == intersect.upperCut:
                            # If equal on right cutpoint, subtract out right
                            # part
                            self.upper_cuts[i] = intersect.lowerCut
                            self.ranges[i] = Range(self.lower_cuts[i], intersect.lowerCut)
                        else:
                            # If in the middle, split into two parts, putting both into
                            # add queue and placing the old range index into the removal
                            # queue
                            addRanges.append(Range(self.lower_cuts[i], intersect.lowerCut))
                            addRanges.append(Range(intersect.upperCut, self.upper_cuts[i]))
                            removeRanges.append(i)
                except ValueError:
                    # Continue if no overlap with this range
                    continue
            # Remove any ranges that are marked for removal
            while len(removeRanges) > 0:
                removeInd = removeRanges.pop()
                self.ranges.pop(removeInd)
                self.lower_cuts.pop(removeInd)
                self.upper_cuts.pop(removeInd)
            # Add any ranges that need to be added
            while len(addRanges) > 0:
                self.add(addRanges.pop())
    def union(self, otherSet):
        """ Creates a new RangeSet that is the union of this set and
        another RangeSet object

        Parameters
        ----------
        otherSet : RangeSet object
            The RangeSet used for the union

        Raises
        ------
        TypeError
            If the object passed in is not a RangeSet
        ValueError
            If the value type of the set not compatible with the ranges

        Returns
        -------
        RangeSet consisting of union of two sets
        """
        if not isinstance(otherSet, RangeSet):
            raise TypeError("otherSet is not a RangeSet")
        return RangeSet(set(self.ranges+otherSet.ranges))



import unittest
from Ranger.src.Range.Range import Range

debug = False

class RangeTest(unittest.TestCase):
    """ Unit Tests for Range.py """
    def test_closed(self):
        if debug: print("Testing closed")
        # Floats
        floatRange = Range.closed(2.,5.)
        self.assertFalse(floatRange.contains(1.99))
        self.assertTrue(floatRange.contains(2.))
        self.assertTrue(floatRange.contains(3.))
        self.assertTrue(floatRange.contains(5.))
        self.assertFalse(floatRange.contains(5.01))
        # Integers
        intRange = Range.closed(2,5)
        self.assertTrue(intRange.contains(2))
        self.assertFalse(intRange.contains(1))
        # Letters
        letterRange = Range.closed('b','e')
        self.assertFalse(letterRange.contains('a'))
        self.assertTrue(letterRange.contains('b'))
        self.assertTrue(letterRange.contains('c'))
        self.assertTrue(letterRange.contains('e'))
        self.assertFalse(letterRange.contains('f'))
    def test_closedOpen(self):
        if debug: print("Testing closedOpen")
        # Floats
        floatRange = Range.closedOpen(2.,5.)
        self.assertFalse(floatRange.contains(1.99))
        self.assertTrue(floatRange.contains(2.))
        self.assertTrue(floatRange.contains(3.))
        self.assertFalse(floatRange.contains(5.))
        self.assertFalse(floatRange.contains(5.01))
        # Letters
        letterRange = Range.closedOpen('b','e')
        self.assertFalse(letterRange.contains('a'))
        self.assertTrue(letterRange.contains('b'))
        self.assertTrue(letterRange.contains('c'))
        self.assertFalse(letterRange.contains('e'))
        self.assertFalse(letterRange.contains('f'))
    def test_openClosed(self):
        if debug: print("Testing openClosed")
        # Floats
        floatRange = Range.openClosed(2.,5.)
        self.assertFalse(floatRange.contains(1.99))
        self.assertFalse(floatRange.contains(2.))
        self.assertTrue(floatRange.contains(3.))
        self.assertTrue(floatRange.contains(5.))
        self.assertFalse(floatRange.contains(5.01))
        # Letters
        letterRange = Range.openClosed('b','e')
        self.assertFalse(letterRange.contains('a'))
        self.assertFalse(letterRange.contains('b'))
        self.assertTrue(letterRange.contains('c'))
        self.assertTrue(letterRange.contains('e'))
        self.assertFalse(letterRange.contains('f'))
    def test_open(self):
        if debug: print("Testing open")
        # Floats
        floatRange = Range.open(2.,5.)
        self.assertFalse(floatRange.contains(1.99))
        self.assertFalse(floatRange.contains(2.))
        self.assertTrue(floatRange.contains(3.))
        self.assertFalse(floatRange.contains(5.))
        self.assertFalse(floatRange.contains(5.01))
        with self.assertRaises(TypeError):
            Range.open(3.,3.)
        # Letters
        letterRange = Range.open('b','e')
        self.assertFalse(letterRange.contains('a'))
        self.assertFalse(letterRange.contains('b'))
        self.assertTrue(letterRange.contains('c'))
        self.assertFalse(letterRange.contains('e'))
        self.assertFalse(letterRange.contains('f'))
        with self.assertRaises(TypeError):
            Range.open('b','b')
    def test_lessThan(self):
        if debug: print("Testing lessThan")
        # Floats
        floatRange = Range.lessThan(5.)
        self.assertTrue(floatRange.contains(1.99))
        self.assertTrue(floatRange.contains(2.))
        self.assertTrue(floatRange.contains(3.))
        self.assertFalse(floatRange.contains(5.))
        self.assertFalse(floatRange.contains(5.01))
        # Letters
        letterRange = Range.lessThan('e')
        self.assertTrue(letterRange.contains('a'))
        self.assertTrue(letterRange.contains('b'))
        self.assertTrue(letterRange.contains('c'))
        self.assertFalse(letterRange.contains('e'))
        self.assertFalse(letterRange.contains('f'))
    def test_atMost(self):
        if debug: print("Testing atMost")
        # Floats
        floatRange = Range.atMost(5.)
        self.assertTrue(floatRange.contains(1.99))
        self.assertTrue(floatRange.contains(2.))
        self.assertTrue(floatRange.contains(3.))
        self.assertTrue(floatRange.contains(5.))
        self.assertFalse(floatRange.contains(5.01))
        # Letters
        letterRange = Range.atMost('e')
        self.assertTrue(letterRange.contains('a'))
        self.assertTrue(letterRange.contains('b'))
        self.assertTrue(letterRange.contains('c'))
        self.assertTrue(letterRange.contains('e'))
        self.assertFalse(letterRange.contains('f'))
    def test_greaterThan(self):
        if debug: print("Testing greaterThan")
        # Floats
        floatRange = Range.greaterThan(5.)
        self.assertFalse(floatRange.contains(1.99))
        self.assertFalse(floatRange.contains(2.))
        self.assertFalse(floatRange.contains(3.))
        self.assertFalse(floatRange.contains(5.))
        self.assertTrue(floatRange.contains(5.01))
        # Letters
        letterRange = Range.greaterThan('e')
        self.assertFalse(letterRange.contains('a'))
        self.assertFalse(letterRange.contains('b'))
        self.assertFalse(letterRange.contains('c'))
        self.assertFalse(letterRange.contains('e'))
        self.assertTrue(letterRange.contains('f'))
    def test_atLeast(self):
        if debug: print("Testing atLeast")
        # Floats
        floatRange = Range.atLeast(5.)
        self.assertFalse(floatRange.contains(1.99))
        self.assertFalse(floatRange.contains(2.))
        self.assertFalse(floatRange.contains(3.))
        self.assertTrue(floatRange.contains(5.))
        self.assertTrue(floatRange.contains(5.01))
        # Letters
        letterRange = Range.atLeast('e')
        self.assertFalse(letterRange.contains('a'))
        self.assertFalse(letterRange.contains('b'))
        self.assertFalse(letterRange.contains('c'))
        self.assertTrue(letterRange.contains('e'))
        self.assertTrue(letterRange.contains('f'))
    def test_hasLowerBound(self):
        if debug: print("Testing hasLowerBound")
        self.assertTrue(Range.atLeast(5.).hasLowerBound())
        self.assertFalse(Range.atMost(5.).hasLowerBound())
        self.assertTrue(Range.closed(2.,5.).hasLowerBound())
    def test_hasUpperBound(self):
        if debug: print("Testing hasUpperBound")
        self.assertFalse(Range.atLeast(5.).hasUpperBound())
        self.assertTrue(Range.atMost(5.).hasUpperBound())
        self.assertTrue(Range.closed(2.,5.).hasUpperBound())
    def test_lowerEndpoint(self):
        if debug: print("Testing lowerEndpoint")
        with self.assertRaises(TypeError):
            Range.atMost(5.).lowerEndpoint()
        self.assertEqual(Range.atLeast(5.).lowerEndpoint(),5.)
        self.assertEqual(Range.closed(5.,10.).lowerEndpoint(),5.)
    def test_upperEndpoint(self):
        if debug: print("Testing upperEndpoint")
        with self.assertRaises(TypeError):
            Range.atLeast(5.).upperEndpoint()
        self.assertEqual(Range.atMost(5.).upperEndpoint(),5.)
        self.assertEqual(Range.closed(5.,10.).upperEndpoint(),10.)
    def test_isLowerBoundClosed(self):
        if debug: print("Testing isLowerBoundClosed")
        with self.assertRaises(TypeError):
            Range.atMost(5.).isLowerBoundClosed()
        self.assertTrue(Range.closedOpen(5.,10.).isLowerBoundClosed())
        self.assertFalse(Range.openClosed(5.,10.).isLowerBoundClosed())
    def test_isUpperBoundClosed(self):
        if debug: print("Testing isUpperBoundClosed")
        with self.assertRaises(TypeError):
            Range.atLeast(5.).isUpperBoundClosed()
        self.assertFalse(Range.closedOpen(5.,10.).isUpperBoundClosed())
        self.assertTrue(Range.openClosed(5.,10.).isUpperBoundClosed())
    def test_isEmpty(self):
        if debug: print("Testing isEmpty")
        self.assertTrue(Range.closedOpen(3.,3.).isEmpty())
        self.assertTrue(Range.openClosed(3.,3.).isEmpty())
        self.assertFalse(Range.openClosed(3.,3.001).isEmpty())
    def test_containsAll(self):
        if debug: print("Testing containsAll")
        self.assertTrue(Range.openClosed(3.,5.).containsAll([4.,4.5,5.]))
        self.assertFalse(Range.closedOpen(3.,5.).containsAll([4.,4.5,5.]))
        self.assertFalse(Range.closedOpen(3.,5.).containsAll([3.,4.,5.]))
        self.assertFalse(Range.closedOpen(3.,5.).containsAll([2.,4.,5.]))
    def test_encloses(self):
        if debug: print("Testing encloses")
        range1 = Range.closed(3.,6.)
        range2 = Range.closed(4.,5.)
        self.assertTrue(range1.encloses(range2))
        range1 = Range.open(3.,6.)
        range2 = Range.open(3.,6.)
        self.assertTrue(range1.encloses(range2))
        range2 = Range.closed(4.,4.)
        self.assertTrue(range1.encloses(range2))
        range1 = Range.openClosed(3.,6.)
        range2 = Range.closed(3.,6.)
        self.assertFalse(range1.encloses(range2))
        self.assertTrue(range2.encloses(range1))
        range1 = Range.closed(4.,5.)
        range2 = Range.open(3.,6.)
        self.assertFalse(range1.encloses(range2))
        self.assertTrue(range2.encloses(range1))
    def test_isConnected(self):
        if debug: print("Testing isConnected")
        range1 = Range.closed(2.,4.)
        range2 = Range.closed(5.,7.)
        self.assertFalse(range1.isConnected(range2))
        range2 = Range.closed(3.,5.)
        self.assertTrue(range1.isConnected(range2))
        range2 = Range.closed(4.,6.)
        self.assertTrue(range1.isConnected(range2))
    def test_intersection(self):
        if debug: print("Testing intersection")
        range1 = Range.closed(1.,5.)
        range2 = Range.closed(3.,7.)
        self.assertEqual(range1.intersection(range2),
                         Range.closed(3.,5.))
        range2 = Range.closed(5.,7.)
        self.assertEqual(range1.intersection(range2),
                         Range.closed(5.,5.))
        range2 = Range.closed(6.,7.)
        with self.assertRaises(ValueError):
            range1.intersection(range2)
    def test_span(self):
        if debug: print("Testing span")
        range1 = Range.closed(1.,3.)
        range2 = Range.closed(5.,7.)
        self.assertEqual(range1.span(range2),
                         Range.closed(1.,7.))
        range2 = Range.closed(2.,5.)
        self.assertEqual(range1.span(range2),
                         Range.closed(1.,5.))
    def test_getDistanceFromPoint(self):
        if debug: print("Testing getDistanceFromPoint")
        range1 = Range.closed(1.,3.)
        self.assertAlmostEqual(range1.getDistanceFromPoint(1.),0.)
        self.assertAlmostEqual(range1.getDistanceFromPoint(2.),0.)
        self.assertAlmostEqual(range1.getDistanceFromPoint(3.),0.)
        self.assertAlmostEqual(range1.getDistanceFromPoint(100.),97.)
        self.assertAlmostEqual(range1.getDistanceFromPoint(0.99),0.01)
        range1 = Range.openClosed(1.,3.)
        with self.assertRaises(TypeError):
            range1.getDistanceFromPoint(0.99)
    def test_getDistanceFromRange(self):
        if debug: print("Testing getDistanceFromRange")
        range1 = Range.closed(1.,3.)
        self.assertAlmostEqual(range1.getDistanceFromRange(Range.closed(5.,7.)),2.)
        self.assertAlmostEqual(range1.getDistanceFromRange(Range.closed(-1.,0.)),1.)
        self.assertAlmostEqual(range1.getDistanceFromRange(Range.closed(-5.,10.)),0.)
        self.assertAlmostEqual(range1.getDistanceFromRange(Range.closed(-5.,2.)),0.)
        self.assertAlmostEqual(range1.getDistanceFromRange(Range.closed(2.,10.)),0.)
        self.assertAlmostEqual(range1.getDistanceFromRange(Range.closed(1.5,2.1)),0.)
        with self.assertRaises(TypeError):
            range1.getDistanceFromRange(Range.closedOpen(1.5,2.1))
if __name__ == "__main__":
    debug = True
    unittest.main(exit = False)
